#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Windows.Forms;

namespace HeuristicLab.Optimization.Networks.Views {
  [View("ServiceParameter View")]
  [Content(typeof(ServiceParameter<>), true)]
  [Content(typeof(IServiceParameter<>), false)]
  [Content(typeof(IServiceParameter), false)]
  public partial class ServiceParameterView : EntityView {
    protected TypeSelectorDialog typeSelectorDialog;

    public new IServiceParameter Content {
      get { return (IServiceParameter)base.Content; }
      set { base.Content = value; }
    }

    public ServiceParameterView() {
      InitializeComponent();
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (typeSelectorDialog != null) typeSelectorDialog.Dispose();
        if (components != null) components.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void DeregisterContentEvents() {
      Content.TypeChanged -= Content_TypeChanged;
      Content.ValueChanged -= Content_ValueChanged;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.TypeChanged += Content_TypeChanged;
      Content.ValueChanged += Content_ValueChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        dataTypeTextBox.Text = string.Empty;
        typeComboBox.SelectedIndex = 0;
        valueViewHost.Content = null;
      } else {
        dataTypeTextBox.Text = Content.DataType.GetPrettyName();
        typeComboBox.SelectedIndex = Content.Type == ServiceParameterType.Input ? 0 : 1;
        valueViewHost.ViewType = null;
        valueViewHost.Content = Content.Value;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      dataTypeTextBox.Enabled = Content != null;
      typeComboBox.Enabled = Content != null && !ReadOnly;
      valueGroupBox.Enabled = Content != null;
      setValueButton.Enabled = Content != null && !ReadOnly;
      clearValueButton.Enabled = Content != null && Content.Value != null && !ReadOnly;
    }
    protected virtual void Content_TypeChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_TypeChanged), sender, e);
      else {
        typeComboBox.SelectedIndex = Content.Type == ServiceParameterType.Input ? 0 : 1;
      }
    }
    protected virtual void Content_ValueChanged(object sender, System.EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ValueChanged), sender, e);
      else {
        clearValueButton.Enabled = Content.Value != null;
        valueViewHost.ViewType = null;
        valueViewHost.Content = Content.Value;
      }
    }

    protected virtual void typeComboBox_SelectedIndexChanged(object sender, EventArgs e) {
      Content.Type = typeComboBox.SelectedIndex == 0 ? ServiceParameterType.Input : ServiceParameterType.Output;
    }

    protected virtual void setValueButton_Click(object sender, EventArgs e) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Value";
      }
      typeSelectorDialog.TypeSelector.Configure(Content.DataType, false, true);
      if (typeSelectorDialog.ShowDialog(this) == DialogResult.OK) {
        try {
          Content.Value = (IItem)typeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
        }
        catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(this, ex);
        }
      }
    }
    protected virtual void clearValueButton_Click(object sender, EventArgs e) {
      Content.Value = null;
    }
    protected virtual void valuePanel_DragEnterOver(object sender, DragEventArgs e) {
      e.Effect = DragDropEffects.None;
      if (!ReadOnly && (e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) is IItem)) {
        if ((e.KeyState & 32) == 32) e.Effect = DragDropEffects.Link;  // ALT key
        else if ((e.KeyState & 4) == 4) e.Effect = DragDropEffects.Move;  // SHIFT key
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Copy)) e.Effect = DragDropEffects.Copy;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Move)) e.Effect = DragDropEffects.Move;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Link)) e.Effect = DragDropEffects.Link;
      }
    }
    protected virtual void valuePanel_DragDrop(object sender, DragEventArgs e) {
      if (e.Effect != DragDropEffects.None) {
        IItem item = e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) as IItem;
        if (e.Effect.HasFlag(DragDropEffects.Copy)) item = (IItem)item.Clone();
        Content.Value = item;
      }
    }
  }
}
